package main

import (
	"container/list"
	"fmt"
)

const (
	HC_MODE_NORMAL = iota
	HC_MODE_STOP
	HC_MODE_THROTTLE
)

type Cache struct {
	mode      int
	size      int64
	used      int64
	total     int64
	lastUsage int
	queue     *list.List
	wait      []int
}

var cache Cache

func (c *Cache) init(opts *Options) {
	c.size = opts.HistoryCacheSize
	c.used = 0
	c.mode = HC_MODE_NORMAL
	c.wait = make([]int, len(opts.Proxies))
	c.queue = list.New()
}

func (c *Cache) usage() int {
	return int(float64(c.used) / float64(c.size) * 100)
}

func (c *Cache) realUsage() int {
	return c.lastUsage
}

func (c *Cache) pauseProxy(index int) {
	if c.wait[index] == 0 {
		c.queue.PushBack(index)
		c.wait[index] = 1
		fmt.Printf("[proxy %02d] paused\n", index)
	}
}

func (c *Cache) resumeProxy(index int) bool {
	e := c.queue.Front()
	if e.Value.(int) != index {
		return false
	}
	c.queue.Remove(e)
	c.wait[index] = 0
	fmt.Printf("[proxy %02d] resumed\n", index)
	return true
}

func (c *Cache) checkProxy(index int) bool {
	if c.mode == HC_MODE_STOP {
		c.pauseProxy(index)
		if c.realUsage() > options.ThrottlePerc {
			return false
		}
		c.mode = HC_MODE_THROTTLE
	} else {
		if c.realUsage() >= options.StopPerc {
			c.mode = HC_MODE_STOP
			c.pauseProxy(index)
			return false
		}
	}

	if c.queue.Len() == 0 {
		return true
	}

	return c.resumeProxy(index)
}

func (c *Cache) add(size int64) bool {
	if c.used+size < c.size {
		c.used += size
		c.total += size
		return true
	}
	return false
}

type CacheSync struct {
	nextcheck int64
}

func (c *CacheSync) scheduled() int64 {
	return c.nextcheck
}

func (c *CacheSync) do() {
	cache.used -= options.ValueSize * options.NVPS
	if cache.used < 0 {
		cache.used = 0
	}
	export.writeData("history.size", int64(float64(cache.used)/float64(cache.size)*100))
	export.writeData("history.total", cache.total)
	fmt.Printf("[history ] used: %d (%.1f), total: %d\n", cache.used,
		float64(cache.used)/float64(cache.size)*100, cache.total)
}

func (h *CacheSync) next() Event {
	return &CacheSync{
		nextcheck: now + 1,
	}
}
