package main

import (
	"encoding/json"
	"fmt"
	"os"
	"time"
)

type ExportMask uint

func (m *ExportMask) String() string {
	return fmt.Sprintf("0x%x", *m)
}

func (m *ExportMask) Set(value string) error {
	switch value {
	case "discovery":
		*m |= EXPORT_DISCOVERY
	case "history":
		*m |= EXPORT_HISTORY
	default:
		return fmt.Errorf("Unknown export mode: %s", value)
	}
	return nil
}

const (
	EXPORT_DISCOVERY = 1 << iota
	EXPORT_HISTORY
)

type Export struct {
	mask   ExportMask
	output *os.File
}

func newExport(mask ExportMask, output string) *Export {
	e := &Export{
		mask: mask,
	}

	if e.mask != 0 {
		var err error
		e.output, err = os.Create(output)
		if err != nil {
			panic(err)
		}
	}
	return e
}

func (e *Export) close() {
	if e.mask != 0 {
		e.output.Close()
	}
}

func (e *Export) writeDiscovery() {
	if (e.mask & EXPORT_DISCOVERY) == 0 {
		return
	}

	data := make([]map[string]string, 0)

	for i := range options.Proxies {
		row := make(map[string]string)
		row["{#PROXY.INDEX}"] = fmt.Sprintf("%d", i)
		data = append(data, row)
	}

	b, err := json.Marshal(&data)
	if err != nil {
		panic(err)
	}

	now := time.Now().Unix()
	_, err = e.output.WriteString(fmt.Sprintf("- proxy.discovery %d %s\n", now, string(b)))
	if err != nil {
		panic(err)
	}
}

func (e *Export) writeData(key string, value int64) {
	if (export.mask & EXPORT_HISTORY) == 0 {
		return
	}
	_, err := e.output.WriteString(fmt.Sprintf("- %s %d %d\n", key, now, value))
	if err != nil {
		panic(err)
	}
}
