package main

import (
	"fmt"
	"math"
)

type Proxy struct {
	cacheSize int64
	startSize int64
	index     int
	opts      *ProxyOptions
}

func newProxy(index int, opts *ProxyOptions) *Proxy {
	return &Proxy{
		cacheSize: opts.HistorySize,
		startSize: opts.HistorySize,
		index:     index,
		opts:      opts,
	}
}

func (p *Proxy) newUploadEvent(now int64) Event {
	size := options.ValueSize * 10000
	if size > p.cacheSize {
		size = p.cacheSize
	}
	if size == 0 {
		return nil
	}
	return &ProxyUpload{
		nextcheck: now + int64(math.Ceil(float64(size)/5.5/float64(p.opts.NetworkSpeed))),
		size:      size,
		proxy:     p,
	}
}

func (p *Proxy) report() {
	export.writeData(fmt.Sprintf("proxy.history.size[%d]", p.index), p.cacheSize)
	export.writeData(fmt.Sprintf("proxy.history.read[%d]", p.index), p.startSize-p.cacheSize)
}

type ProxyUpload struct {
	nextcheck int64
	size      int64
	proxy     *Proxy
}

func (u *ProxyUpload) scheduled() int64 {
	return u.nextcheck
}

func (u *ProxyUpload) do() {
	if !cache.checkProxy(u.proxy.index) {
		u.proxy.report()
		return
	}
	if cache.add(u.size) {
		u.proxy.cacheSize -= u.size
		if u.proxy.cacheSize < 0 {
			u.proxy.cacheSize = 0
		}
		fmt.Printf("[proxy %02d] cache: %d\n", u.proxy.index, u.proxy.cacheSize)
	} else {
		panic(fmt.Sprintf("[proxy %02d] dropped\n", u.proxy.index))
	}
	u.proxy.report()
}

func (u *ProxyUpload) next() Event {
	return u.proxy.newUploadEvent(now)
}
